package com.gic.haoban.manage.service.service.out.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.contacts.manage.api.dto.SyncJsonColumnDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.dao.mapper.DepartmentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanStaffDepartmentRelated;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.StaffDepartmentRelatedService;
import com.github.pagehelper.PageHelper;

@Service
public class DepartmentApiServiceImpl implements DepartmentApiService {
	
	private static Logger logger  = LoggerFactory.getLogger(DepartmentApiServiceImpl.class);
	
	@Autowired
	private DepartmentService departmentService;
	@Autowired
	private StaffDepartmentRelatedService staffDepartmentService;
	
	@Autowired
	private DepartmentMapper departmentMapper;
	
	@Autowired
	private com.gic.enterprise.api.service.DepartmentService gicDepartmentService;
	
	@Override
	public List<DepartmentDTO> listByParentId(String parentId) {
		List<TabHaobanDepartment> list = departmentService.listByParentId(parentId);
		
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public DepartmentDTO selectById(String departmentId) {
		TabHaobanDepartment tab = departmentService.selectById(departmentId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public HaobanResponse add(DepartmentDTO department) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		//调微信的新增接口
		String wxDepartmentId = "";
		if(StringUtils.isBlank(wxDepartmentId)) {
			hr.setErrorCode(0);
			hr.setMessage("微信新增部门失败");
			return hr;
		}
		department.setWxDepartmentId(wxDepartmentId);
		String departmentId = departmentService.add(department);
		department.setDepartmentId(departmentId);
		hr.setResult(department);
		return hr;
	}

	@Override
	public HaobanResponse edit(DepartmentDTO department) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		//调微信的修改接口
		String wxDepartmentId = "";
		if(StringUtils.isBlank(wxDepartmentId)) {
			hr.setErrorCode(0);
			hr.setMessage("微信修改部门失败");
			return hr;
		}
		
		departmentService.edit(department);
		return hr;
	}

	@Override
	public HaobanResponse del(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		//调微信的新增接口
		String wxDepartmentId = "";
		if(StringUtils.isBlank(wxDepartmentId)) {
			hr.setErrorCode(0);
			hr.setMessage("微信删除部门失败");
			return hr;
		}
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		if(!list.isEmpty()) {
			hr.setErrorCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		if(!staffList.isEmpty()) {
			hr.setErrorCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		
		departmentService.del(departmentId);
		return hr;
	}

	@Override
	public DepartmentDTO selectByRelatedId(String relatedId) {
		TabHaobanDepartment tab = departmentService.selectByRelatedId(relatedId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public HaobanResponse recycle(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		departmentService.recycle(departmentId);
		return hr;
	}

	@Override
	public Page<DepartmentDTO> pageDepartmentByParams(BasePageInfo pageInfo, String wxEnterpriseId, String keyword,
			Integer storeFlag, Integer recycleFlag) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return  PageUtil.changePageHelperToCurrentPage(departmentService.pageDepartmentByParams(wxEnterpriseId,keyword,storeFlag,recycleFlag),DepartmentDTO.class);
	}

	@Override
	public HaobanResponse repairRecycle(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(0);
		departmentService.repairRecycle(departmentId);
		return hr;
		
	}

	@Override
	public List<DepartmentDTO> listByDepartmentIds(List<String> departmentIds,
			int storeFlag) {
		List<TabHaobanDepartment> list= departmentMapper.listByDepartmentIds(departmentIds, storeFlag);
		if(list==null){
			return new ArrayList<DepartmentDTO>();
		}
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public DepartmentDTO getRootByEnterpriseId(String wxEnterpriseId) {
		
		return departmentService.getRootByEnterpriseId(wxEnterpriseId);
	}

	@Override
	public List<DepartmentDTO> listStoreByWxEnterpriseId(String wxEnterpriseId) {
		List<TabHaobanDepartment> list = departmentMapper.listStoreByWxEnterpriseId(wxEnterpriseId);
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public Page<DepartmentDTO> pageFullStoreByWxEnterpriseId(
			String wxEnterpriseId,String search, int count,BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return PageUtil.changePageHelperToCurrentPage( departmentMapper.pageFullStoreByWxEnterpriseId(wxEnterpriseId,search,count),DepartmentDTO.class);
	}

	@Override
	public void syncGicStore(String fieldListString) {
		logger.info("【同步门店】fieldList={}",fieldListString);
		List<SyncJsonColumnDTO> fieldList = JSONArray.parseArray(fieldListString,SyncJsonColumnDTO.class);
		if(fieldList.isEmpty()){
	        return;
	    }
		
		Integer status = 0;
		String storeName = "";
		String departmentId = "";
		String storeId = "";
		for (SyncJsonColumnDTO syncJsonColumnDTO:fieldList){
	    	String name = StringUtil.camelName(syncJsonColumnDTO.getName());
	        String value = syncJsonColumnDTO.getValue();
	        if("status".equals(name)) {
	        	status = Integer.parseInt(value);
	        }
	        if("storeName".equals(name)) {
	        	storeName = value;
	        }
	        if("departmentId".equals(name)) {
	        	departmentId = value;
	        }
	        if("storeId".equals(name)) {
	        	storeId = value;
	        }
		}
		DepartmentDTO department = departmentService.getByRelatedId(storeId);
		if(department == null) {
			if(status != null && status == 2) {
				DepartmentDTO parent = getParent(departmentId);
				if(parent == null) {
//					WxEen
//					parent = getRootByEnterpriseId(wxEnterpriseId)；
				}
			}
		}
		
	        
		
	}
	/**
	 * 向上找父节点
	 * @param departmentId
	 * @return
	 */
	private DepartmentDTO getParent(String departmentId) {
		DepartmentDTO department = departmentService.getByRelatedId(departmentId);
		if(department == null) {
			com.gic.enterprise.api.dto.DepartmentDTO dto = gicDepartmentService.getDeptment(departmentId);
			String parentId = dto.getParentDepartId();
			getParent(parentId);
		}
		return department;
	}

	@Override
	public void syncGicDepartment(String fieldListString) {
		// TODO Auto-generated method stub
		
	}

}
